import 'dart:convert';

import 'package:json_annotation/json_annotation.dart';
import 'package:lori_sdk/data/models/cart/paymentMethod.dart';
import 'package:lori_sdk/data/models/userAddress.dart';

part 'user.g.dart';

final String tableName = 'user';
final String columnId = 'id';
final String columnName = 'full_name';
final String columnEmail = 'email';
final String columnImageUrl = 'image_url';
final String columnCpf = 'cpf';
final String columnPhoneNumber = 'phone_number';
final String columnDateOfBirth = 'date_of_birth';
final String columnGender = 'gender';
final String columnPayments = 'payments';
final String columnAddresses = 'addresses';

@JsonSerializable()
class User {
  User();

  String id;
  String fullName;
  String email;
  String imageUrl;
  String cpf;
  String phoneNumber;
  String dateOfBirth;
  int gender;
  List<PaymentMethod> paymentMethods;
  List<UserAddress> addresses;

  factory User.fromJson(Map<String, dynamic> json) => _$UserFromJson(json);

  Map<String, dynamic> toJson() => _$UserToJson(this);

  // Helpers method to support SQLite
  Map<String, dynamic> toMap() {
    var map = <String, dynamic>{
      columnId: id,
      columnName: fullName,
      columnEmail: email,
      columnImageUrl: imageUrl,
      columnCpf: cpf,
      columnPhoneNumber: phoneNumber,
      columnDateOfBirth: dateOfBirth,
      columnGender: gender,
      columnPayments: json.encode(paymentMethods),
      columnAddresses: json.encode(addresses),
    };
    return map;
  }

  void fromMap(Map<String, dynamic> map) {
    id = map[columnId];
    fullName = map[columnName];
    email = map[columnEmail];
    imageUrl = map[columnImageUrl];
    cpf = map[columnCpf];
    phoneNumber = map[columnPhoneNumber];
    dateOfBirth = map[columnDateOfBirth];
    gender = map[columnGender];

    Iterable iPayments = json.decode(map[columnPayments]);
    List<PaymentMethod> parsedPaymentMethods = [];
    iPayments.forEach((map) {
      parsedPaymentMethods.add(PaymentMethod.fromJson(map));
    });
    paymentMethods = parsedPaymentMethods;

    Iterable iAddresses = json.decode(map[columnAddresses]);
    List<UserAddress> parsedAddresses = [];
    iAddresses.forEach((map) {
      parsedAddresses.add(UserAddress.fromJson(map));
    });
    addresses = parsedAddresses;
  }
}
